@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.82 2021/10/12 09:27:21 tobhe Exp $	*/
+/*	$OpenBSD: config.c,v 1.85 2022/05/08 20:26:31 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2019-2021 Tobias Heider <tobhe@openbsd.org>
@@ -615,7 +615,7 @@ int
 config_getsocket(struct iked *env, struct imsg *imsg,
     void (*cb)(int, short, void *))
 {
-	struct iked_socket	*sock, **sock0, **sock1;
+	struct iked_socket	*sock, **sock0 = NULL, **sock1 = NULL;
 
 	log_debug("%s: received socket fd %d", __func__, imsg->fd);
 
@@ -925,6 +925,8 @@ config_getstatic(struct iked *env, struct imsg *imsg)
 	log_debug("%s: %sstickyaddress", __func__,
 	    env->sc_stickyaddress ? "" : "no ");
 
+	ikev2_reset_alive_timer(env);
+
 	return (0);
 }
 
@@ -1094,7 +1096,5 @@ config_getkey(struct iked *env, struct imsg *imsg)
 	explicit_bzero(imsg->data, len);
 	ca_getkey(&env->sc_ps, &id, imsg->hdr.type);
 
-	ikev2_reset_alive_timer(env);
-
 	return (0);
 }
