@@ -1,4 +1,4 @@
-/*	$OpenBSD: pfkey.c,v 1.77 2021/03/02 03:31:25 jsg Exp $	*/
+/*	$OpenBSD: pfkey.c,v 1.80 2021/11/25 19:41:03 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2020-2021 Tobias Heider <tobhe@openbsd.org>
@@ -128,17 +128,17 @@ static const struct pfkey_constmap pfkey_satype[] = {
 };
 
 int	pfkey_map(const struct pfkey_constmap *, uint16_t, uint8_t *);
-int	pfkey_flow(int, uint8_t, uint8_t, struct iked_flow *);
-int	pfkey_sa(int, uint8_t, uint8_t, struct iked_childsa *);
-int	pfkey_sa_getspi(int, uint8_t, struct iked_childsa *, uint32_t *);
-int	pfkey_sagroup(int, uint8_t, uint8_t,
+int	pfkey_flow(struct iked *, uint8_t, uint8_t, struct iked_flow *);
+int	pfkey_sa(struct iked *, uint8_t, uint8_t, struct iked_childsa *);
+int	pfkey_sa_getspi(struct iked *, uint8_t, struct iked_childsa *, uint32_t *);
+int	pfkey_sagroup(struct iked *, uint8_t, uint8_t,
 	    struct iked_childsa *, struct iked_childsa *);
-int	pfkey_write(int, struct sadb_msg *, struct iovec *, int,
+int	pfkey_write(struct iked *, struct sadb_msg *, struct iovec *, int,
 	    uint8_t **, ssize_t *);
 int	pfkey_reply(int, uint8_t **, ssize_t *);
 void	pfkey_dispatch(int, short, void *);
-int	pfkey_sa_lookup(int, struct iked_childsa *, uint64_t *);
-int	pfkey_sa_check_exists(int, struct iked_childsa *);
+int	pfkey_sa_lookup(struct iked *, struct iked_childsa *, uint64_t *);
+int	pfkey_sa_check_exists(struct iked *, struct iked_childsa *);
 
 struct sadb_ident *
 	pfkey_id2ident(struct iked_id *, unsigned int);
@@ -148,7 +148,7 @@ void	pfkey_timer_cb(int, short, void *);
 int	pfkey_process(struct iked *, struct pfkey_message *);
 
 int
-pfkey_couple(int sd, struct iked_sas *sas, int couple)
+pfkey_couple(struct iked *env, struct iked_sas *sas, int couple)
 {
 	struct iked_sa		*sa;
 	struct iked_flow	*flow;
@@ -156,7 +156,7 @@ pfkey_couple(int sd, struct iked_sas *sas, int couple)
 	const char		*mode[] = { "coupled", "decoupled" };
 
 	/* Socket is not ready */
-	if (sd == -1)
+	if (env->sc_pfkey == -1)
 		return (-1);
 
 	if (sadb_decoupled == !couple)
@@ -171,21 +171,21 @@ pfkey_couple(int sd, struct iked_sas *sas, int couple)
 	RB_FOREACH(sa, iked_sas, sas) {
 		TAILQ_FOREACH(csa, &sa->sa_childsas, csa_entry) {
 			if (!csa->csa_loaded && couple)
-				(void)pfkey_sa_add(sd, csa, NULL);
+				(void)pfkey_sa_add(env, csa, NULL);
 			else if (csa->csa_loaded && !couple)
-				(void)pfkey_sa_delete(sd, csa);
+				(void)pfkey_sa_delete(env, csa);
 			if ((ipcomp = csa->csa_bundled) != NULL) {
 				if (!ipcomp->csa_loaded && couple)
-					(void)pfkey_sa_add(sd, ipcomp, csa);
+					(void)pfkey_sa_add(env, ipcomp, csa);
 				else if (ipcomp->csa_loaded && !couple)
-					(void)pfkey_sa_delete(sd, ipcomp);
+					(void)pfkey_sa_delete(env, ipcomp);
 			}
 		}
 		TAILQ_FOREACH(flow, &sa->sa_flows, flow_entry) {
 			if (!flow->flow_loaded && couple)
-				(void)pfkey_flow_add(sd, flow);
+				(void)pfkey_flow_add(env, flow);
 			else if (flow->flow_loaded && !couple)
-				(void)pfkey_flow_delete(sd, flow);
+				(void)pfkey_flow_delete(env, flow);
 		}
 	}
 
@@ -208,7 +208,7 @@ pfkey_map(const struct pfkey_constmap *map, uint16_t alg, uint8_t *pfkalg)
 }
 
 int
-pfkey_flow(int sd, uint8_t satype, uint8_t action, struct iked_flow *flow)
+pfkey_flow(struct iked *env, uint8_t satype, uint8_t action, struct iked_flow *flow)
 {
 #ifdef __OpenBSD__
 	struct sadb_msg		 smsg;
@@ -477,7 +477,7 @@ pfkey_flow(int sd, uint8_t satype, uint8_t action, struct iked_flow *flow)
 
 #undef PAD
 
-	ret = pfkey_write(sd, &smsg, iov, iov_cnt, NULL, NULL);
+	ret = pfkey_write(env, &smsg, iov, iov_cnt, NULL, NULL);
 
 	free(sa_srcid);
 	free(sa_dstid);
@@ -487,7 +487,7 @@ pfkey_flow(int sd, uint8_t satype, uint8_t action, struct iked_flow *flow)
 	struct sadb_msg		 smsg;
 	struct sadb_address	 sa_src, sa_dst;
 	struct sadb_x_ipsecrequest sa_ipsec;
-	struct sadb_x_policy	 sa_policy, *sa_polid;
+	struct sadb_x_policy	 sa_policy;
 	struct sadb_x_sa2	 sa_2;
 	struct sockaddr_storage  ssrc, sdst, slocal, speer;
 	struct iovec		 iov[IOV_CNT];
@@ -495,8 +495,6 @@ pfkey_flow(int sd, uint8_t satype, uint8_t action, struct iked_flow *flow)
 	in_port_t		 sport, dport;
 	uint8_t			 smask, dmask;
 	uint8_t			 zeropad[8];
-	uint8_t			*reply = NULL;
-	ssize_t			 rlen;
 	uint64_t		 pad = 0;
 	size_t			 padlen;
 
@@ -666,20 +664,7 @@ pfkey_flow(int sd, uint8_t satype, uint8_t action, struct iked_flow *flow)
 
 #undef PAD
 
-	ret = -1;
-	if (pfkey_write(sd, &smsg, iov, iov_cnt, &reply, &rlen) != 0)
-		goto done;
-
-	if ((sa_polid = pfkey_find_ext(reply, rlen,
-	    SADB_X_EXT_POLICY)) == NULL) {
-		log_debug("%s: erroneous reply", __func__);
-		goto done;
-	}
-
-	ret = 0;
-
- done:
-	free(reply);
+	ret = pfkey_write(env, &smsg, iov, iov_cnt, NULL, NULL);
 
 #endif /* __OpenBSD__ */
 
@@ -687,7 +672,7 @@ pfkey_flow(int sd, uint8_t satype, uint8_t action, struct iked_flow *flow)
 }
 
 int
-pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
+pfkey_sa(struct iked *env, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 {
 	struct sadb_msg		 smsg;
 	struct sadb_sa		 sadb;
@@ -977,6 +962,7 @@ pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 	if (satype == SADB_X_SATYPE_IPCOMP)
 		sadb.sadb_sa_encrypt = SADB_X_CALG_DEFLATE;
 
+#if defined __OpenBSD__
 	/* Note that we need to swap the IDs for incoming SAs (SADB_UPDATE) */
 	if (action != SADB_UPDATE) {
 		sa_srcid = pfkey_id2ident(
@@ -989,6 +975,7 @@ pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 		sa_dstid = pfkey_id2ident(
 		    IKESA_SRCID(sa->csa_ikesa), SADB_EXT_IDENTITY_DST);
 	}
+#endif
 
 #ifdef SADB_X_EXT_TAG
 	tag = sa->csa_ikesa->sa_tag;
@@ -1146,6 +1133,7 @@ pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 		PAD(ibuf_size(sa->csa_integrkey));
 	}
 
+#if defined __OpenBSD__
 	if (sa_srcid) {
 		/* src identity */
 		iov[iov_cnt].iov_base = sa_srcid;
@@ -1160,6 +1148,7 @@ pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 		smsg.sadb_msg_len += sa_dstid->sadb_ident_len;
 		iov_cnt++;
 	}
+#endif
 
 #ifdef SADB_X_EXT_TAG
 	if (tag != NULL) {
@@ -1196,7 +1185,7 @@ pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 
 #undef PAD
 
-	ret = pfkey_write(sd, &smsg, iov, iov_cnt, NULL, NULL);
+	ret = pfkey_write(env, &smsg, iov, iov_cnt, NULL, NULL);
 
 	free(sa_srcid);
 	free(sa_dstid);
@@ -1205,7 +1194,7 @@ pfkey_sa(int sd, uint8_t satype, uint8_t action, struct iked_childsa *sa)
 }
 
 int
-pfkey_sa_lookup(int sd, struct iked_childsa *sa, uint64_t *last_used)
+pfkey_sa_lookup(struct iked *env, struct iked_childsa *sa, uint64_t *last_used)
 {
 	struct sadb_msg		*msg, smsg;
 	struct sadb_address	 sa_src, sa_dst;
@@ -1333,7 +1322,7 @@ pfkey_sa_lookup(int sd, struct iked_childsa *sa, uint64_t *last_used)
 	}
 #endif
 
-	if ((ret = pfkey_write(sd, &smsg, iov, iov_cnt, &data, &n)) != 0)
+	if ((ret = pfkey_write(env, &smsg, iov, iov_cnt, &data, &n)) != 0)
 		return (-1);
 
 	msg = (struct sadb_msg *)data;
@@ -1366,19 +1355,19 @@ done:
 }
 
 int
-pfkey_sa_last_used(int sd, struct iked_childsa *sa, uint64_t *last_used)
+pfkey_sa_last_used(struct iked *env, struct iked_childsa *sa, uint64_t *last_used)
 {
-	return pfkey_sa_lookup(sd, sa, last_used);
+	return pfkey_sa_lookup(env, sa, last_used);
 }
 
 int
-pfkey_sa_check_exists(int sd, struct iked_childsa *sa)
+pfkey_sa_check_exists(struct iked *env, struct iked_childsa *sa)
 {
-	return pfkey_sa_lookup(sd, sa, NULL);
+	return pfkey_sa_lookup(env, sa, NULL);
 }
 
 int
-pfkey_sa_getspi(int sd, uint8_t satype, struct iked_childsa *sa,
+pfkey_sa_getspi(struct iked *env, uint8_t satype, struct iked_childsa *sa,
     uint32_t *spip)
 {
 	struct sadb_msg		*msg, smsg;
@@ -1476,7 +1465,7 @@ pfkey_sa_getspi(int sd, uint8_t satype, struct iked_childsa *sa,
 
 	*spip = 0;
 
-	if ((ret = pfkey_write(sd, &smsg, iov, iov_cnt, &data, &n)) != 0)
+	if ((ret = pfkey_write(env, &smsg, iov, iov_cnt, &data, &n)) != 0)
 		return (-1);
 
 	msg = (struct sadb_msg *)data;
@@ -1502,7 +1491,7 @@ done:
 
 #ifdef __OpenBSD__
 int
-pfkey_sagroup(int sd, uint8_t satype1, uint8_t action,
+pfkey_sagroup(struct iked *env, uint8_t satype1, uint8_t action,
     struct iked_childsa *sa1, struct iked_childsa *sa2)
 {
 	struct sadb_msg		smsg;
@@ -1664,15 +1653,16 @@ pfkey_sagroup(int sd, uint8_t satype1, uint8_t action,
 
 #undef PAD
 
-	return (pfkey_write(sd, &smsg, iov, iov_cnt, NULL, NULL));
+	return (pfkey_write(env, &smsg, iov, iov_cnt, NULL, NULL));
 }
 #endif
 
 int
-pfkey_write(int sd, struct sadb_msg *smsg, struct iovec *iov, int iov_cnt,
+pfkey_write(struct iked *env, struct sadb_msg *smsg, struct iovec *iov, int iov_cnt,
     uint8_t **datap, ssize_t *lenp)
 {
 	ssize_t n, len = smsg->sadb_msg_len * 8;
+	int ret = -1;
 
 	if (sadb_decoupled) {
 		switch (smsg->sadb_msg_type) {
@@ -1689,21 +1679,27 @@ pfkey_write(int sd, struct sadb_msg *smsg, struct iovec *iov, int iov_cnt,
 		}
 	}
 
-	if ((n = writev(sd, iov, iov_cnt)) == -1) {
+	/* Delete event to poll() in pfkey_reply() */
+	event_del(&env->sc_pfkeyev);
+
+	if ((n = writev(env->sc_pfkey, iov, iov_cnt)) == -1) {
 		log_warn("%s: writev failed: type %u len %zd",
 		    __func__, smsg->sadb_msg_type, len);
-		return (-1);
+		goto done;
 	} else if (n != len) {
 		log_warn("%s: short write", __func__);
-		return (-1);
+		goto done;
 	}
 
-	return (pfkey_reply(sd, datap, lenp));
+	ret = pfkey_reply(env->sc_pfkey, datap, lenp);
+ done:
+	event_add(&env->sc_pfkeyev, NULL);
+	return (ret);
 }
 
 /* wait for pfkey response and returns 0 for ok, -1 for error, -2 for timeout */
 int
-pfkey_reply(int sd, uint8_t **datap, ssize_t *lenp)
+pfkey_reply(int fd, uint8_t **datap, ssize_t *lenp)
 {
 	struct pfkey_message	*pm;
 	struct sadb_msg		 hdr;
@@ -1712,7 +1708,7 @@ pfkey_reply(int sd, uint8_t **datap, ssize_t *lenp)
 	struct pollfd		pfd[1];
 	int			 n;
 
-	pfd[0].fd = sd;
+	pfd[0].fd = fd;
 	pfd[0].events = POLLIN;
 
 	for (;;) {
@@ -1733,7 +1729,7 @@ pfkey_reply(int sd, uint8_t **datap, ssize_t *lenp)
 			return (-2);	/* retry */
 		}
 
-		if (recv(sd, &hdr, sizeof(hdr), MSG_PEEK) != sizeof(hdr)) {
+		if (recv(fd, &hdr, sizeof(hdr), MSG_PEEK) != sizeof(hdr)) {
 			log_warn("%s: short recv", __func__);
 			return (-1);
 		}
@@ -1750,7 +1746,7 @@ pfkey_reply(int sd, uint8_t **datap, ssize_t *lenp)
 		}
 		len = hdr.sadb_msg_len * PFKEYV2_CHUNK;
 
-		if (read(sd, data, len) != len) {
+		if (read(fd, data, len) != len) {
 			log_warnx("%s: short read", __func__);
 			free(data);
 			return (-1);
@@ -1801,7 +1797,7 @@ pfkey_reply(int sd, uint8_t **datap, ssize_t *lenp)
 }
 
 int
-pfkey_flow_add(int fd, struct iked_flow *flow)
+pfkey_flow_add(struct iked *env, struct iked_flow *flow)
 {
 	uint8_t		 satype;
 
@@ -1811,7 +1807,7 @@ pfkey_flow_add(int fd, struct iked_flow *flow)
 	if (pfkey_map(pfkey_satype, flow->flow_saproto, &satype) == -1)
 		return (-1);
 
-	if (pfkey_flow(fd, satype, SADB_X_ADDFLOW, flow) == -1)
+	if (pfkey_flow(env, satype, SADB_X_ADDFLOW, flow) == -1)
 		return (-1);
 
 	flow->flow_loaded = 1;
@@ -1820,7 +1816,7 @@ pfkey_flow_add(int fd, struct iked_flow *flow)
 }
 
 int
-pfkey_flow_delete(int fd, struct iked_flow *flow)
+pfkey_flow_delete(struct iked *env, struct iked_flow *flow)
 {
 	uint8_t		satype;
 
@@ -1830,7 +1826,7 @@ pfkey_flow_delete(int fd, struct iked_flow *flow)
 	if (pfkey_map(pfkey_satype, flow->flow_saproto, &satype) == -1)
 		return (-1);
 
-	if (pfkey_flow(fd, satype, SADB_X_DELFLOW, flow) == -1)
+	if (pfkey_flow(env, satype, SADB_X_DELFLOW, flow) == -1)
 		return (-1);
 
 	flow->flow_loaded = 0;
@@ -1839,14 +1835,14 @@ pfkey_flow_delete(int fd, struct iked_flow *flow)
 }
 
 int
-pfkey_sa_init(int fd, struct iked_childsa *sa, uint32_t *spi)
+pfkey_sa_init(struct iked *env, struct iked_childsa *sa, uint32_t *spi)
 {
 	uint8_t		 satype;
 
 	if (pfkey_map(pfkey_satype, sa->csa_saproto, &satype) == -1)
 		return (-1);
 
-	if (pfkey_sa_getspi(fd, satype, sa, spi) == -1)
+	if (pfkey_sa_getspi(env, satype, sa, spi) == -1)
 		return (-1);
 
 	log_debug("%s: new spi 0x%08x", __func__, *spi);
@@ -1855,7 +1851,7 @@ pfkey_sa_init(int fd, struct iked_childsa *sa, uint32_t *spi)
 }
 
 int
-pfkey_sa_add(int fd, struct iked_childsa *sa, struct iked_childsa *last)
+pfkey_sa_add(struct iked *env, struct iked_childsa *sa, struct iked_childsa *last)
 {
 	uint8_t		 satype;
 	unsigned int	 cmd;
@@ -1872,25 +1868,25 @@ pfkey_sa_add(int fd, struct iked_childsa *sa, struct iked_childsa *last)
 	log_debug("%s: %s spi %s", __func__, cmd == SADB_ADD ? "add": "update",
 	    print_spi(sa->csa_spi.spi, 4));
 
-	rval = pfkey_sa(fd, satype, cmd, sa);
+	rval = pfkey_sa(env, satype, cmd, sa);
 	if (rval != 0) {
 		if (cmd == SADB_ADD) {
 			if (rval == -2) {
 				/* timeout: check for existence */
-				if (pfkey_sa_check_exists(fd, sa) == 0) {
+				if (pfkey_sa_check_exists(env, sa) == 0) {
 					log_debug("%s: SA exists after timeout",
 					    __func__);
 					goto loaded;
 				}
 			}
-			(void)pfkey_sa_delete(fd, sa);
+			(void)pfkey_sa_delete(env, sa);
 			return (-1);
 		}
 		if (sa->csa_allocated && !sa->csa_loaded && errno == ESRCH) {
 			/* Needed for recoupling local SAs */
 			log_debug("%s: SADB_UPDATE on local SA returned ESRCH,"
 			    " trying SADB_ADD", __func__);
-			if (pfkey_sa(fd, satype, SADB_ADD, sa) == -1)
+			if (pfkey_sa(env, satype, SADB_ADD, sa) == -1)
 				return (-1);
 		} else {
 			return (-1);
@@ -1900,9 +1896,9 @@ pfkey_sa_add(int fd, struct iked_childsa *sa, struct iked_childsa *last)
  loaded:
 	if (last != NULL) {
 #ifdef __OpenBSD__
-		if (pfkey_sagroup(fd, satype,
+		if (pfkey_sagroup(env, satype,
 		    SADB_X_GRPSPIS, sa, last) == -1) {
-			(void)pfkey_sa_delete(fd, sa);
+			(void)pfkey_sa_delete(env, sa);
 			return (-1);
 		}
 #endif
@@ -1913,7 +1909,7 @@ pfkey_sa_add(int fd, struct iked_childsa *sa, struct iked_childsa *last)
 }
 
 int
-pfkey_sa_update_addresses(int fd, struct iked_childsa *sa)
+pfkey_sa_update_addresses(struct iked *env, struct iked_childsa *sa)
 {
 	uint8_t		 satype;
 
@@ -1927,11 +1923,11 @@ pfkey_sa_update_addresses(int fd, struct iked_childsa *sa)
 	if (pfkey_map(pfkey_satype, sa->csa_saproto, &satype) == -1)
 		return (-1);
 	log_debug("%s: spi %s", __func__, print_spi(sa->csa_spi.spi, 4));
-	return pfkey_sa(fd, satype, IKED_SADB_UPDATE_SA_ADDRESSES, sa);
+	return pfkey_sa(env, satype, IKED_SADB_UPDATE_SA_ADDRESSES, sa);
 }
 
 int
-pfkey_sa_delete(int fd, struct iked_childsa *sa)
+pfkey_sa_delete(struct iked *env, struct iked_childsa *sa)
 {
 	uint8_t		satype;
 
@@ -1941,8 +1937,8 @@ pfkey_sa_delete(int fd, struct iked_childsa *sa)
 	if (pfkey_map(pfkey_satype, sa->csa_saproto, &satype) == -1)
 		return (-1);
 
-	if (pfkey_sa(fd, satype, SADB_DELETE, sa) == -1 &&
-	    pfkey_sa_check_exists(fd, sa) == 0)
+	if (pfkey_sa(env, satype, SADB_DELETE, sa) == -1 &&
+	    pfkey_sa_check_exists(env, sa) == 0)
 		return (-1);
 
 	sa->csa_loaded = 0;
@@ -1950,7 +1946,7 @@ pfkey_sa_delete(int fd, struct iked_childsa *sa)
 }
 
 int
-pfkey_flush(int sd)
+pfkey_flush(struct iked *env)
 {
 	struct sadb_msg smsg;
 	struct iovec	iov[IOV_CNT];
@@ -1970,7 +1966,7 @@ pfkey_flush(int sd)
 	iov[iov_cnt].iov_len = sizeof(smsg);
 	iov_cnt++;
 
-	return (pfkey_write(sd, &smsg, iov, iov_cnt, NULL, NULL));
+	return (pfkey_write(env, &smsg, iov, iov_cnt, NULL, NULL));
 }
 
 struct sadb_ident *
@@ -2023,7 +2019,7 @@ pfkey_id2ident(struct iked_id *id, unsigned int exttype)
 }
 
 int
-pfkey_socket(void)
+pfkey_socket(struct iked *env)
 {
 	int	 fd;
 
@@ -2033,8 +2029,6 @@ pfkey_socket(void)
 	if ((fd = socket(PF_KEY, SOCK_RAW, PF_KEY_V2)) == -1)
 		fatal("pfkey_socket: failed to open PF_KEY socket");
 
-	pfkey_flush(fd);
-
 	return (fd);
 }
 
@@ -2057,6 +2051,8 @@ pfkey_init(struct iked *env, int fd)
 	    EV_READ|EV_PERSIST, pfkey_dispatch, env);
 	event_add(&env->sc_pfkeyev, NULL);
 
+	pfkey_flush(env);
+
 	/* Register it to get ESP and AH acquires from the kernel */
 	bzero(&smsg, sizeof(smsg));
 	smsg.sadb_msg_version = PF_KEY_V2;
@@ -2069,7 +2065,7 @@ pfkey_init(struct iked *env, int fd)
 	iov.iov_base = &smsg;
 	iov.iov_len = sizeof(smsg);
 
-	if (pfkey_write(fd, &smsg, &iov, 1, NULL, NULL))
+	if (pfkey_write(env, &smsg, &iov, 1, NULL, NULL))
 		fatal("pfkey_init: failed to set up ESP acquires");
 
 	bzero(&smsg, sizeof(smsg));
@@ -2083,7 +2079,7 @@ pfkey_init(struct iked *env, int fd)
 	iov.iov_base = &smsg;
 	iov.iov_len = sizeof(smsg);
 
-	if (pfkey_write(fd, &smsg, &iov, 1, NULL, NULL))
+	if (pfkey_write(env, &smsg, &iov, 1, NULL, NULL))
 		fatal("pfkey_init: failed to set up AH acquires");
 }
 
@@ -2104,7 +2100,7 @@ pfkey_find_ext(uint8_t *data, ssize_t len, int type)
 }
 
 void
-pfkey_dispatch(int sd, short event, void *arg)
+pfkey_dispatch(int fd, short event, void *arg)
 {
 	struct iked		*env = (struct iked *)arg;
 	struct pfkey_message	 pm, *pmp;
@@ -2112,7 +2108,7 @@ pfkey_dispatch(int sd, short event, void *arg)
 	ssize_t			 len;
 	uint8_t			*data;
 
-	if (recv(sd, &hdr, sizeof(hdr), MSG_PEEK) != sizeof(hdr)) {
+	if (recv(fd, &hdr, sizeof(hdr), MSG_PEEK) != sizeof(hdr)) {
 		log_warn("%s: short recv", __func__);
 		return;
 	}
@@ -2129,7 +2125,7 @@ pfkey_dispatch(int sd, short event, void *arg)
 	}
 	len = hdr.sadb_msg_len * PFKEYV2_CHUNK;
 
-	if (read(sd, data, len) != len) {
+	if (read(fd, data, len) != len) {
 		log_warn("%s: short read", __func__);
 		free(data);
 		return;
@@ -2204,7 +2200,7 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 	struct sadb_x_policy	*sa_pol;
 #endif
 	struct iovec		 iov[IOV_CNT];
-	int			 ret = 0, iov_cnt, sd;
+	int			 ret = 0, iov_cnt;
 	uint8_t			*reply;
 	ssize_t			 rlen;
 	uint8_t			*data = pm->pm_data;
@@ -2214,7 +2210,6 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 	if (!env || !data || !len)
 		return (0);
 
-	sd = env->sc_pfkey;
 	hdr = (struct sadb_msg *)data;
 
 	switch (hdr->sadb_msg_type) {
@@ -2271,7 +2266,7 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 		smsg.sadb_msg_len += sizeof(sa_pol) / 8;
 		iov_cnt++;
 
-		if (pfkey_write(sd, &smsg, iov, iov_cnt, &reply, &rlen)) {
+		if (pfkey_write(env, &smsg, iov, iov_cnt, &reply, &rlen)) {
 			log_warnx("%s: failed to get a policy", __func__);
 			return (0);
 		}
@@ -2432,7 +2427,7 @@ out:
 		iov[iov_cnt].iov_len = sizeof(*sa_pol);
 		iov_cnt++;
 
-		if (pfkey_write(sd, &smsg, iov, iov_cnt, &reply, &rlen)) {
+		if (pfkey_write(env, &smsg, iov, iov_cnt, &reply, &rlen)) {
 			log_warnx("%s: failed to get a policy", __func__);
 			break;
 		}
